Package: atime
Type: Package
Title: Asymptotic Timing
Version: 2026.4.2
Depends: R (>= 3.5.0)
Authors@R: c(
    person("Toby", "Hocking",
     email="toby.hocking@r-project.org",
     role=c("aut", "cre")))
Description: Computing and visualizing comparative
 asymptotic timings of different algorithms and code versions.
 Also includes functionality for comparing empirical timings with
 expected references such as linear or quadratic,
 <https://en.wikipedia.org/wiki/Asymptotic_computational_complexity>
 Also includes functionality for measuring asymptotic memory and other
 quantities.
License: GPL-3
URL: https://github.com/tdhock/atime, https://atime-docs.netlify.app
BugReports: https://github.com/tdhock/atime/issues
Imports: data.table, bench, lattice, git2r, utils, stats, grDevices
Suggests: directlabels, ggplot2, testthat, stringi, re2, binsegRcpp,
        grates, fastymd, wbs, fpop, changepoint, LOPART, PeakSegDisk,
        callr, readr, dplyr, tidyr, nc, RColorBrewer, Matrix, litedown
VignetteBuilder: litedown
NeedsCompilation: no
Packaged: 2026-04-04 03:37:10 UTC; hoct2726
Author: Toby Hocking [aut, cre]
Maintainer: Toby Hocking <toby.hocking@r-project.org>
Repository: CRAN
Date/Publication: 2026-04-04 16:20:02 UTC
Built: R 4.5.2; ; 2026-04-08 01:53:10 UTC; unix
