importFrom("utils", "capture.output", "install.packages", "packageVersion")
importFrom("stats", "median", "quantile", "sd", "approx", "predict")
import(data.table)
import(lattice)
export(atime)
export(atime_grid)
export(references_best)
export(atime_versions)
export(atime_versions_remove)
export(atime_versions_exprs)
export(atime_pkg)
export(atime_pkg_test_info)
export(atime_test)
export(atime_test_list)
export(glob_find_replace)
S3method("print", "atime")
S3method("plot", "atime")
S3method("predict", "references_best")
S3method("print", "references_best")
S3method("plot", "references_best")
S3method("print", "atime_prediction")
S3method("plot", "atime_prediction")
